'''This contains classes for the different context types for translation rules'''
# Copyright (C) 2007 Michael Whapples, All Rights Reserved.
# Original Author: Michael Whapples, mwhapples@users.sourceforge.net
# Unless linsor has explicitly granted you any other licensing terms, the contents of this file is subject to the RPL 1.1,
# available at the BrlTex website (http://brltex.sourceforge.net).
# All software distributed under the Licenses is provided strictly on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND the licensor
# HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT, OR NON-INFRINGEMENT.
# Licensor reserves the right to change the Venue stated in the license to the UK.
# 

import re
import string
import sys
try:
	from psyco.classes import *
except:
	pass


class ContextSuper:
	def __init__(self, pattern):
		self.pattern = pattern

class ContextA(ContextSuper):
	def __init__(self, pattern):
		ContextSuper.__init__(self, pattern)
		self.priority = 0
	def search(self, context):
		return True

class ContextCSuper(ContextSuper):
	def __init__(self, pattern):
		ContextSuper.__init__(self, pattern)
		self.priority = 2

class ContextCBefore(ContextCSuper):
	def search(self, context):
		return context[-1] in self.pattern

class ContextCAfter(ContextCSuper):
	def search(self, context):
		return context[0] in self.pattern

class ContextTSuper(ContextSuper):
	def __init__(self, pattern):
		ContextSuper.__init__(self, pattern)
		self.priority = 3

class ContextTBefore(ContextTSuper):
	def search(self, context):
		return context.endswith(self.pattern)

class ContextTAfter(ContextTSuper):
	def search(self, context):
		return context.startswith(self.pattern)

class ContextR(ContextSuper):
	def __init__(self, pattern):
		self.pattern = pattern
		self.compiled = re.compile(pattern)
		self.priority = 1
	def search(self, context):
		if self.compiled.search(context):
			return True
		else:
			return False

class ContextTypeFactory:
	def __init__(self):
		self.__customMatches = {}
	def getContext(self, type, pattern, before):
		if type == '^r':
			if before == True:
				contextObject = ContextR(pattern + '$')
			else:
				contextObject = ContextR('^' + pattern)
		elif type == '^a':
			contextObject = ContextA(pattern)
		elif type == '^t':
			if before:
				contextObject = ContextTBefore(pattern)
			else:
				contextObject = ContextTAfter(pattern)
		elif type == '^c':
			if before:
				contextObject = ContextCBefore(self.__customMatches[pattern])
			else:
				contextObject=ContextCAfter(self.__customMatches[pattern])
		#We should check that the search function is actually valid
		#This need not actually match, so long as it doesn't give an error
		try:
			contextObject.search(u'h')
			return contextObject
		except:
			pass
	def setCustomMatches(self, cmatches):
		self.__customMatches = cmatches
	def getCustomMatches(self):
		return self.__customMatches




'''
Module containing the base translation system

In this module you are meant to only use the Translator class.
'''
# Copyright (C) 2007 Michael Whapples, All Rights Reserved.
# Original Author: Michael Whapples, mwhapples@users.sourceforge.net
# Unless linsor has explicitly granted you any other licensing terms, the contents of this file is subject to the RPL 1.1,
# available at the BrlTex website (http://brltex.sourceforge.net).
# All software distributed under the Licenses is provided strictly on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND the licensor
# HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT, OR NON-INFRINGEMENT.
# Licensor reserves the right to change the Venue stated in the license to the UK.
# 

from pkg_resources import resource_stream
import logging
import array
import sys, os.path
from YABT import contextTypes
from YABT import exceptions
from xml.dom import minidom
from xml.parsers.expat import ExpatError
import StringIO
try:
	from psyco.classes import *
except:
	pass
logging.getLogger("YABT").manager.emittedNoHandlerWarning = True
class TranslationRule:
	'''
	This is the base class for translation rules.
	You are not meant to use this directly, but rather use the methods in the Translator class to create rules.
	'''
	def bFirst(self, bContext, aContext):
		return self.beforeContext(bContext) and self.afterContext(aContext)
	def aFirst(self, bContext, aContext):
		return self.afterContext(aContext) and self.beforeContext(bContext)
	def __init__(self, inputClass, focus, beforeContext, afterContext, translation, finalState, contextFactory):
		self.inputClass = inputClass
		self.focus = focus
		self.beforeContextObj=contextFactory.getContext(beforeContext[:2], beforeContext[2:], True)
		self.beforeContext=self.beforeContextObj.search
		self.afterContextObj=contextFactory.getContext(afterContext[:2], afterContext[2:], False)
		self.afterContext=self.afterContextObj.search
		if self.beforeContextObj.priority > self.afterContextObj.priority:
			self.checkContext=self.bFirst
		else:
			self.checkContext=self.aFirst
		self.translation = translation
		self.finalState = finalState

class Translator:
	'''The base translator class
	
	To make use of this you would do something like the following:
	
	>>> brl = Translator()
	>>> brl.loadConfigFile('britishtobrl.xml')
	>>> brl.translateText("hello world", 2, " ", " ")
	u'HELLO _W'
	
	If you forget to do the loadConfigFile call this will not raise an error, but simply no translation will happen.
	The default no rule translation is to insert the character so we would get:
	
	>>> translator().translateText("hello world", 2, " ", " ")
	u'hello world'
	'''
	def __init__(self):
		self.__characterMaps = {}
		self.__groups = {}
		self.__decission = {}
		self.contextFactory = contextTypes.ContextTypeFactory()
		self.logger = logging.getLogger('YABT.Translator')
		if self.logger.level == 0:
			self.logger.setLevel(logging.WARNING)
		if self.logger.isEnabledFor(logging.INFO):
			self.logger.info('Created a translator object')
	def getNumOfGroups(self):
		return len(self.__groups)
	def __addRule(self, inputClass, focus, beforeContext, afterContext, translation, finalState):
		#create the rule
		rule = TranslationRule(inputClass, focus, beforeContext, afterContext, translation, finalState, self.contextFactory)
		#Create the group if necessary and add the rule
		if not self.__groups.has_key(rule.focus[0]):
			self.__groups.update({rule.focus[0]: []})
			if self.logger.isEnabledFor(logging.DEBUG):
				self.logger.debug("Added the group " + focus[0])
		self.__groups[rule.focus[0]].append(rule)
		if self.logger.isEnabledFor(logging.DEBUG):
			self.logger.debug("Created rule "+str(inputClass)+","+focus+","+beforeContext+","+afterContext+","+translation+","+str(finalState))
	def __addConnection(self, inputClass, state):
		# Add the inputClass if no decissions have be created for it before
		if not self.__decission.has_key(inputClass):
			self.__decission.update({inputClass: []})
			if self.logger.isEnabledFor(logging.DEBUG):
				self.logger.debug("Added input class " + str(inputClass))
		# Add the state to the inputClass
		self.__decission[inputClass].append(state)
		if self.logger.isEnabledFor(logging.DEBUG):
			self.logger.debug("Added state " + str(state) + " to " + str(inputClass))
	def __addCharacterMap(self, original, replacement):
		self.__characterMaps.update({original: replacement})
		if self.logger.isEnabledFor(logging.DEBUG):
			self.logger.debug("Added the character mapping from " + original + " to " + replacement)
	def getStates(self):
		states = []
		for inputClass in self.__decission.values():
			for state in inputClass:
				if not state in states:
					states.append(state)
		states.sort()
		return states
	def __noGroupTranslation(self, translateString, currentPosition):
		if self.logger.isEnabledFor(logging.DEBUG):
			self.logger.debug("Character error " + translateString[currentPosition])
		return self.noGroupHandler(translateString, currentPosition)
	def noGroupHandler(self, translateString, currentPosition):
		return translateString[currentPosition]
	def __noRuleToApply(self, translateString, currentPosition):
		if self.logger.isEnabledFor(logging.DEBUG):
			self.logger.debug("Rule error: cannot match anything around " + translateString[currentPosition] + " in state " + str(self.state))
		return self.noRuleHandler(translateString, currentPosition)
	def noRuleHandler(self, translateString, currentPosition):
		return translateString[currentPosition]
	def __applyRule(self, translateString, currentPosition, ruleToApply):
		if self.logger.isEnabledFor(logging.DEBUG):
			self.logger.debug('translating from ' + ruleToApply.focus + ' in state ' + str(self.state) + ' with ' + ruleToApply.translation)
		if ruleToApply.finalState >= 0:
			self.state = ruleToApply.finalState
		return ruleToApply.translation
	def __applyCharacterMap(self, translateString):
		charMaps = self.__characterMaps
		for origChar, replChar in charMaps.iteritems():
			translateString = translateString.replace(origChar, replChar)
		return translateString
	def __beforeContextMatches(self, beforeString, beforeContext):
		return beforeContext(beforeString)
	def __afterContextMatches(self, afterString, afterContext):
		return afterContext(afterString)
	def __ruleMatches(self, translateString, currentPosition, endOfTranslateString, rules):
		# Set up some variables needed by the loop
		ruleNum = 0
		groupLen = len(rules)
		decissions = self.__decission
		state = self.state
		beforeString = translateString[:currentPosition]
		remainingString = translateString[currentPosition:]
		endPos = endOfTranslateString - currentPosition
		while ruleNum < groupLen:
			rule = rules[ruleNum]
			#first check if the rule is in the inputClass
			if state in decissions[rule.inputClass]:
				#now check the focus matches
				if remainingString.startswith(rule.focus, 0, endPos):
					#Check the context
					if rule.checkContext(beforeString, remainingString[len(rule.focus):]):
						#Seems like the rule is valid
						return rule
			ruleNum += 1
		return None
	def __translateText(self, textToTranslate, state, fakeBeforeText, fakeAfterText):
		#set everything up
		self.state = state
		currentPosition = len(fakeBeforeText)
		translateString = fakeBeforeText + textToTranslate + fakeAfterText
		#apply the character mappings
		translateString = self.__applyCharacterMap(translateString)
		#loop over just the textToTranslte part of translateString
		getGroups = self.__groups
		ruleMatches = self.__ruleMatches
		applyRule = self.__applyRule
		translateStringLen = len(translateString)
		translateTo = translateStringLen-len(fakeAfterText)
		#outputList = array.array('u')
		#outputAppend = outputList.fromunicode
		outputList = StringIO.StringIO()
		outputAppend = outputList.write
		while currentPosition < translateTo:
			#get the rules of the group beginning with current character
			try:
				group = getGroups[translateString[currentPosition]]
			except:
				manualTranslateString = self.__noGroupTranslation(translateString, currentPosition)
				outputAppend(manualTranslateString)
				currentPosition += len(manualTranslateString)
				continue
			#Check the rules in group for a match
			ruleToApply = ruleMatches(translateString, currentPosition, translateTo, group)
			if not ruleToApply is None:
				outputAppend(applyRule(translateString, currentPosition, ruleToApply))
				currentPosition = currentPosition + len(ruleToApply.focus)
			else:
				manualTranslateString = self.__noRuleToApply(translateString, currentPosition)
				outputAppend(manualTranslateString)
				currentPosition = currentPosition + len(manualTranslateString)
		output = outputList.getvalue()
		outputList.close()
		return output
	def translateText(self, translateString, state, beforeContext=' ', afterContext=' ', buffered=True, bufferChar='\f'):
		if not state in self.getStates():
			raise exceptions.InvalidStateException("State not defined")
		sys.setcheckinterval(5000)
		translated = u''
		if buffered:
			inputBuffers = [beforeContext]
			inputBuffers.extend(translateString.split(bufferChar))
			inputBuffers.append(afterContext)
			#counter = 2
			self.state = state
			inputLen = len(inputBuffers)
			#prevBuffer = beforeContext
			#curBuffer = inputBuffers[1]
			outputList = [self.__translateText(inputBuffers[i-1], self.state, inputBuffers[i -2], inputBuffers[i]) for i in xrange(2,inputLen)]
			#while counter < len(inputBuffers):
			#	outputList.append(self.__translateText(curBuffer, self.state, prevBuffer, inputBuffers[counter]))
			#	prevBuffer = curBuffer
			#	curBuffer = inputBuffers[counter]
			#	counter+=1
			translated=bufferChar.join(outputList)
		else:
			translated = self.__translateText(translateString, state, beforeContext, afterContext)
		sys.setcheckinterval(100)
		return translated
	def loadConfigFile(self, fileName):
		'''
		This loads a YABT configuration table from a file
		'''
		try:
			# First get the DOM
			configdoc = minidom.parse(fileName).documentElement
		except ExpatError:
			raise exceptions.InvalidTranslationTableException("The file does not seem to be valid XML")
		if self.logger.isEnabledFor(logging.INFO):
			self.logger.info("Translation table " + fileName + " is valid XML")
		self.__loadConfig(configdoc)
	def loadConfigString(self, configString):
		try:
			configdoc = minidom.parseString(configString).documentElement
		except ExpatError:
			raise Exceptions.InvalidTranslationTableException("The string is not valid XML of a translation table")
		if self.logger.isEnabledFor(logging.INFO):
			self.logger.info("Translation table is valid XML")
		self.__loadConfig(configdoc)
	def __loadConfig(self, configdoc):
		# Now get the meta data section, should be the first one only
		metadata_section = configdoc.getElementsByTagName('metadata')[0]
		# At the moment we only get the YABT_custom_matches from this
		cmatches = {}
		for node in metadata_section.getElementsByTagName('YABT_custom_matches')[0].childNodes:
			if node.nodeName != '#text':
				cmatches.update({node.nodeName: getTextContent(node)})
		if self.logger.isEnabledFor(logging.DEBUG):
			self.logger.debug(str(len(cmatches)) + " custom character match groups created")
		self.contextFactory.setCustomMatches(cmatches)
		# Now get the YABT section
		yabt_sections = configdoc.getElementsByTagName('YABTdata')
		# We may have more than one for some reason (unknown to me)
		for datasection in yabt_sections:
			# Now go through the nodes deciding what to do
			for node in datasection.childNodes:
				if node.nodeName == 'charmap':
					self.__addCharacterMap(getTextContent(node.childNodes[0]), 
						getTextContent(node.childNodes[1]))
					if self.logger.isEnabledFor(logging.DEBUG):
						self.logger.debug("Added char map from line " + node.toxml())
				elif node.nodeName == 'decission':
					inputclass = node.attributes['inputclass'].childNodes[0].wholeText
					for state in node.attributes['states'].childNodes[0].wholeText.split(','):
						self.__addConnection(int(inputclass), int(state))
					if self.logger.isEnabledFor(logging.DEBUG):
						self.logger.debug("Added decission " + node.toxml())
				if node.nodeName == 'rule':
					self.__addRule(int(node.childNodes[0].childNodes[0].wholeText),
						getTextContent(node.childNodes[1]),
						getTextContent(node.childNodes[2]),
						getTextContent(node.childNodes[3]),
						getTextContent(node.childNodes[4]),
						int(node.childNodes[5].childNodes[0].wholeText))
					if self.logger.isEnabledFor(logging.DEBUG):
						self.logger.debug("Added rule from " + node.toxml())
		if self.logger.isEnabledFor(logging.INFO):
			self.logger.info("Finished loading translation table")

def getTextContent(node):
	'''getTextContent(node) - Gets the text content of a node. Any childNodes with nodeName char will be 
converted to characters of ordinal given by the ord attribute.'''
	outtext=u''
	if not node.hasChildNodes():
		return u''
	for child in node.childNodes:
		if child.nodeName == '#text':
			outtext+=child.nodeValue
		elif child.nodeName == 'char':
			outtext+=unichr(int(child.attributes['ord'].childNodes[0].nodeValue))
	return outtext







'''This module provides the exception classes for YABT'''

class InvalidTranslationTableException(Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr(self.value)

class InvalidStateException(Exception):
	def __init__(self, value):
		self.value = value
	def __str__(self):
		return repr(self.value)




#!/usr/bin/python
import ez_setup
ez_setup.use_setuptools()
from setuptools import setup
import logging
log = logging.root
setup(name='YABT',
version='1.0.1',
author='Michael Whapples',
author_email='mwhapples@users.sourceforge.net',
url='http://brltex.sf.net',
description='YABT: Yet Another Braille Translator',
long_description='''
YABT: Yet another Braille Translator

YABT is a general purpose Braille translation system written in pure python. It is primarily designed to be used by the 
`BrlTex Project <http://brltex.sf.net>`_, but due to its general design it may be suited to use in other projects.

Currently YABT has a table for translation in to British Braille encoded in ASCII Braille, but tables for other codes 
and other output encodings such as unicode Braille are possible.
''',
license="RPL",
classifiers=['Development Status :: 5 - Production/Stable',
	'Intended Audience :: Developers',
	'License :: OSI Approved',
	'Operating System :: OS Independent',
	'Programming Language :: Python',
	'Topic :: Adaptive Technologies',
	'Topic :: Software Development'
],
packages=['YABT'],
package_data={'YABT': ['logging.conf', 'britishtobrl.xml']},
entry_points={'console_scripts':['YABT_translate = YABT.app:standardTranslate']},
test_suite="tests.run_tests"
)




'''This module contains useful functions which may be run over the text before translating it using YABT.'''
# Copyright (C) 2007 Michael Whapples, All Rights Reserved.
# Original Author: Michael Whapples, mwhapples@users.sourceforge.net
# Unless linsor has explicitly granted you any other licensing terms, the contents of this file is subject to the RPL 1.1,
# available at the BrlTex website (http://brltex.sourceforge.net).
# All software distributed under the Licenses is provided strictly on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND the licensor
# HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT, OR NON-INFRINGEMENT.
# Licensor reserves the right to change the Venue stated in the license to the UK.
# 

import string
def addCapMarks(intext):
	'''addCapMarks(intext) - adds capital markers in intext and returns the new string.'''
	# We add two types of capital markers, letter and word given by $CL and $CW
	counter = 0
	outtext = []
	intextlen = len(intext)
	while counter < intextlen:
		counter2 = 0
		while counter2 < len(intext):
			if not intext[counter2] in string.uppercase:
				break
			else:
				counter2 += 1
		if counter2 == 1:
			outtext.append(u'$CL ' + intext[0])
			counter +=1
			intext = intext[1:]
		elif counter2 > 1:
			outtext.append(u'$CW ' + intext[:counter2] + u'$EW ')
			counter += counter2
			intext = intext[counter2:]
		else:
			outtext.append(intext[0])
			counter +=1
			intext = intext[1:]
	return u''.join(outtext)



# Copyright (C) 2007 Michael Whapples, All Rights Reserved.
# Original Author: Michael Whapples, mwhapples@users.sourceforge.net
# Unless linsor has explicitly granted you any other licensing terms, the contents of this file is subject to the RPL 1.1,
# available at the BrlTex website (http://brltex.sourceforge.net).
# All software distributed under the Licenses is provided strictly on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND the licensor
# HEREBY DISCLAIMS ALL SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT, OR NON-INFRINGEMENT.
# Licensor reserves the right to change the Venue stated in the license to the UK.
# 

import sys
import YABT
from pkg_resources import resource_string
def standardTranslate():
	# Check we have two arguments
	if len(sys.argv) != 3:
		print('USAGE: YABT_translate file_to_translate state\nfile_to_translate should be a text file and state should be 1 for grade 1 or 2 for grade 2.')
		sys.exit(2)
	# First load the data in the file specified on the command line
	try:
		f = open(sys.argv[1])
		input_text = unicode(f.read(), 'Latin1')
		f.close()
	except:
		print('There seems to be a problem with the file.')
	# Now do the translation
	mytrans = YABT.Translator()
	mytrans.loadConfigString(resource_string('YABT','tables/britishtobrl.xml'))
	print(mytrans.translateText(input_text, int(sys.argv[2]), ' ', ' ', True).encode('Latin1'))



This is for the module YABT.contextTypes, which is not for general use but is used internally by YABT. It is purely 
being documented here for those who may want to do further development of YABT.

The purpose of this module is to generate Context objects for YABT. If you wish to add a new way for YABT to identify 
the context for the rules then here is the right place to add that functionality.

None of the actual context object classes are accessed directly, so they won't be tested here. All usage should be 
through the ContextTypeFactory class which will generate the correct objects for you.

The usage of the ContextTypeFactory class is as follows

First of all the module needs importing.

>>> from YABT.contextTypes import ContextTypeFactory

Now we can access the class create an object.

>>> contextFactory = ContextTypeFactory()
>>> contextFactory.__class__.__name__
'ContextTypeFactory'

Now we have the object, we may wish to set some values for particular types. At the moment only the custom match type 
requires some values to be set. For this we need to call the setCustomMatches method. This method takes a dictionary, 
where keys are the names of the groups and the values are the characters belonging to the group.

>>> contextFactory.setCustomMatches({'lowercase': 'abcdefghijklmnopqrstuvwxyz'})

Once this has been done, we can start creating the context objects. For the context object creation we use the 
getContext method. This method takes three parameters, the type string, the pattern and a boolean indicating if the 
context is for before (True for before and False for after). A use of the method might be

>>> matchAny = contextFactory.getContext('^a', '', True)

In that case above, the second and third parameters don't actually matter as the ^a type should always match, so is not 
context sensitive. The below shows some of this.

>>> matchAny2 = contextFactory.getContext('^a', 'ffdk', False)
>>> matchAny.search('hello world')
True
>>> matchAny2.search('hello world')
True
>>> matchAny.search(' ')
True
>>> matchAny2.search(' ')
True
>>> matchAny.search('')
True

And we could go on, but it would be pointless as for a ^a context its search method should always return True.

You have already met one context type, but there are others and they are much more interesting.

The next we will introduce, you have heard mentioned earlier in this document, is the custom match or ^c type. This type 
works by having defined groups of characters and tests whether the character adjacent  is a member of the group. The 
pattern for the ^c type is the name of the group of characters. We could do something like the following to demostrate 
this.

>>> lowercaseBefore = contextFactory.getContext('^c', 'lowercase', True)
>>> lowercaseBefore.search('hello')
True
>>> lowercaseBefore.search('.12h')
True
>>> lowercaseBefore.search('123.,')
False

As you can see, the ^c type does not always match, but it is also direction sensitive. We can do the previous on a after 
defined version.

>>> lowercaseAfter = contextFactory.getContext('^c', 'lowercase', False)
>>> lowercaseAfter.search('hello')
True
>>> lowercaseAfter.search('.12h')
False
>>> lowercaseAfter.search('123..,')
False

The remaining types do not need any special set up.

The next type we will introduce is the text match or ^t type. This simply matches the text in the context in the 
direction specified.

>>> textBefore = contextFactory.getContext('^t', 'hello ', True)
>>> textBefore.search('hello ')
True

The match must be exact, so the following doesn't work.

>>> textBefore.search('hello')
False

Neither does

>>> textBefore.search('Say " hello')
False

We also have the same sort of behaviour for the after version of this.

>>> textAfter = contextFactory.getContext('^t', ' hello ', False)
>>> textAfter.search(' hello world')
True
>>> textAfter.search(' hello ')
True
>>> textAfter.search(' hello')
False
>>> textAfter.search('hello world')
False

The remaining type is the regular expression or ^r type. This takes a regular expression as the pattern. The matching on 
this is complicated and is best explained else where where regular expressions are discussed. The only thing you should 
be aware of is that regular expressions have a way to specify whether it should match at the beginning or end of the 
text, but this should not be given in the pattern as it is automatically inserted for you according to the before 
setting. Some examples are given below.

>>> reAfter = contextFactory.getContext('^r', r'[0-9]+\.[0-9]+', False)
>>> reAfter.search('0.5')
True
>>> reAfter.search('90.2113')
True
>>> reAfter.search('1.788965')
True
>>> reAfter.search('.09')
False
>>> reAfter.search('x.098')
False




#!/usr/bin/python

# This script will run the doctests

import os
import sys
import unittest
import doctest

# first of all set the correct directory

rootDir = os.path.join(os.path.dirname(__file__), os.pardir)
#sys.path.insert(0, os.getcwd())
suite = unittest.TestSuite()
# Have a module to run all tests in a directory
def getTests(testDir):
	global suite
	#Set the directories not to be checked for tests
	noTestDirs = ['test_data', '.svn']
	for root, dirs, files in os.walk(testDir):
		for f in files:
			suite.addTest(doctest.DocFileSuite(os.path.join(root, f), module_relative=False))
		for removeDir in noTestDirs:
			if removeDir in dirs:
				dirs.remove(removeDir)

def additional_tests():
	global suite
	# Now actually run some of the tests
	getTests(os.path.join(rootDir, 'tests/docs'))
	getTests(os.path.join(rootDir, 'tests/bugs'))
	#runner = unittest.TextTestRunner()
	#runner.run(suite)
	return suite
